

#property indicator_chart_window

extern int    MaxBars   = 500;        // Bar count to be processed (to save CPU time)
extern double MinFactor = 2;           // Minimum multiple of ATR a breakout candle must have to cause a line

int MaxLines = 2000;
double L[2000], S[2000];
datetime LT[2000], ST[2000];
int Trend;

int init() {
  for(int i = 0; i < MaxLines; i++) {
    L[i] = 0;
    S[i] = 0;
  }
  Trend = 0;
  return(0);
}

int deinit() {
  for(int j = 0; j < MaxLines; j++) {
    string n = "ZL"+j;
    ObjectDelete(n);
    n = "ZS"+j;
    ObjectDelete(n);
  }
  for(j = 0; j < Trend; j++) {
    n = "ZT"+j;
    ObjectDelete(n);
  }
  init();
  return(0);
}

string GetTimeDiffNice(int t) {
  t = Time[0] - t;
  if (t < 120) return(t+" seconds ago");
  t /= 60;
  if (t < 120) return(t+" minutes ago");
  t /= 60;
  if (t < 48) return(t+" hours ago");
  t /= 24;
  return(t+" days ago");
}

int start() {
  static datetime latest = 0;
  int changed = Bars - IndicatorCounted();
  if (changed < 2) return(0);
  
  for(int i = MathMin(changed-1, MaxBars); i > 0; i--) {
    if (latest >= Time[i]) continue;
    latest = Time[i];

    for(int j = 0; j < MaxLines; j++) {
      if ((L[j] > 0) && (Low[i] <= L[j])) {
        string n = "ZT"+Trend;
        ObjectDelete(n);
        ObjectCreate(n, OBJ_TREND, 0, LT[j], L[j], Time[i-1]-Period()*60, L[j]);
        ObjectSet(n, OBJPROP_COLOR, SeaGreen);
        ObjectSet(n, OBJPROP_RAY, false);
        ObjectSet(n, OBJPROP_BACK, true);
        L[j] = 0;
        Trend++;
      }
      if ((S[j] > 0) && (High[i] >= S[j])) {
        n = "ZT"+Trend;
        ObjectDelete(n);
        ObjectCreate(n, OBJ_TREND, 0, ST[j], S[j], Time[i-1]-Period()*60, S[j]);
        ObjectSet(n, OBJPROP_COLOR, Purple);
        ObjectSet(n, OBJPROP_RAY, false);
        ObjectSet(n, OBJPROP_BACK, true);
        S[j] = 0;
        Trend++;
      }
    }

    double r = MathAbs(Close[i] - Open[i]) - MathAbs(Close[i+1] - Open[i+1]);
    //double h = Low[i+1];
    if (Close[i] - Close[i+1] > 0 && High[i] - High[i+1] > 0 && Close[i] - High[i+1] > 0 && r > 0) {
      for(j = 0; j < MaxLines; j++) {
        if (L[j] == High[i+1]) break;
        if (L[j] != 0) continue;
        if (Open[i+1] < Close[i+1]) {L[j] = Close[i+1];} else {L[j] = Open[i+1];}
        LT[j] = Time[i+1];
        break;
      }
    }

    if (Close[i] - Close[i+1] < 0 && Low[i] - Low[i+1] < 0 && Close[i] - Low[i+1] < 0 && r > 0) {
      for(j = 0; j < MaxLines; j++) {
        if (S[j] == Low[i+1]) break;
        if (S[j] != 0) continue;
        if (Open[i+1] < Close[i+1]) {S[j] = Open[i+1];} else {S[j] = Close[i+1];}
        ST[j] = Time[i+1];
        break;
      }
    }
  }
  
  for(j = 0; j < MaxLines; j++) {
    n = "ZL"+j;
    if (L[j] == 0) {
      ObjectDelete(n);
      continue;
    } 
    ObjectCreate(n, OBJ_FIBO, 0, 0, 0);
    ObjectSet(n, OBJPROP_FIBOLEVELS, 1);
    ObjectSet(n, OBJPROP_LEVELCOLOR, YellowGreen);
    ObjectSet(n, OBJPROP_TIME1, LT[j]);
    ObjectSet(n, OBJPROP_PRICE1, L[j]);
    ObjectSet(n, OBJPROP_TIME2, Time[0]);
    ObjectSet(n, OBJPROP_PRICE2, L[j]);
    ObjectSet(n, OBJPROP_BACK, true);
    ObjectSetFiboDescription(n, 0, GetTimeDiffNice(LT[j])+" "+DoubleToStr(L[j], Digits));
  }

  for(j = 0; j < MaxLines; j++) {
    n = "ZS"+j;
    if (S[j] == 0) {
      ObjectDelete(n);
      continue;
    } 
    ObjectCreate(n, OBJ_FIBO, 0, 0, 0);
    ObjectSet(n, OBJPROP_FIBOLEVELS, 1);
    ObjectSet(n, OBJPROP_LEVELCOLOR, Crimson);
    ObjectSet(n, OBJPROP_TIME1, ST[j]);
    ObjectSet(n, OBJPROP_PRICE1, S[j]);
    ObjectSet(n, OBJPROP_TIME2, Time[0]);
    ObjectSet(n, OBJPROP_PRICE2, S[j]);
    ObjectSet(n, OBJPROP_BACK, true);
    ObjectSetFiboDescription(n, 0, GetTimeDiffNice(ST[j])+" "+DoubleToStr(S[j], Digits));
  }

  return(0);
}

